/*
2023-04-14
Michael Nowak
http://www.mino-elektronik.de
Alle Angaben ohne Gewaehr !
*/

#define FM_AS65_BEDIENUNG

#include "System-Pico-FM-AS65.h"
#define TEMP_STR_LEN  30

#define REPEAT_19     19        // Tasten Repeat-Schwellen
#define REPEAT_39     39        // 1,9 3,9 und 5,9 Sekunden
#define REPEAT_59     59
#define INC_DEC_1     1         // Startwert fr +/-1 
#define INC_DEC_5     5         // inc-dec bei Repeat-Tasten
#define INC_DEC_10    10
#define INC_DEC_50    50
#define INC_DEC_100   100
#define INC_DEC_1000  300       // Geschwindigkeit nach Geschmack anpassen

uint8_t entprellte_tasten, akt_tasten;    	// zur externen Verwendung

static uint8_t    temp_tasten, letzte_aenderung, sperre;
static int8_t     rep_flag, entprell_ctr, rep_ctr;
volatile uint8_t  taste;

uint8_t     tast_tab[] = "ABCD    ";      // 1. Tastencode
uint8_t rep_tast_tab[] = "ab X    ";      // repeatcode teilweise zulassen
uint8_t deko_tab[] = {0,1,2,4,8,16,32,64,128};

void menue_F1();
void menue_F2();
void menue_allgemein();

uint8_t lese_taste_x() 
{
uint8_t temp;
  handle_ser_input();
  F1_messung(0);                // immer weiter messen
  sperre = 1;
  temp = taste;                 // Tastencode lesen
  taste=0;                      // und loeschen
  sperre = 0;
  return(temp);
}

uint8_t lese_taste()
{
static uint8_t temp_taste;
uint8_t temp = lese_taste_x();
  if(temp == ABBRUCH_TASTE) {
    rep_flag = 0;               // ohne repeat
    temp_taste = 0;
    return(temp);
  }
  if(temp == INIT_TASTE) {
    temp_taste = temp;          // merken
    return(0);                  // und noch abwarten
  }
  if(temp_taste == INIT_TASTE) {
    if((entprellte_tasten & INIT_BIT) == 0) {  // taste wurde losgelassen
      temp = temp_taste;
      temp_taste = 0;
      return(temp);
    } else temp = 0;            // ignorieren
  }
  return(temp);                 // alle anderen Tasten unverndert lassen
}

void dekodiere_tasten(void)
{
uint8_t n,  tasten_schalter, aenderung;
static uint8_t delay;

  if(lcd_busy) return;  // GPIO nicht lesbar
  
    delay++;
    if(delay < 10 || sperre) return;          // von 1 ms auf 10 ms Intervall oder sperren
    delay = 0;
    SIO->GPIO_OE_CLR = TASTEN_MASKE << TASTEN_OFFSET; // Ausgaenge abschalten
// Pullup einschalten und kleine Verzoegerung    
    PADS_BANK0->GPIO6 = PADS_BANK0_GPIO1_PUE_Msk | PADS_BANK0_GPIO1_IE_Msk | PADS_BANK0_GPIO1_SCHMITT_Msk; // Pullup = 1
    PADS_BANK0->GPIO7 = PADS_BANK0_GPIO1_PUE_Msk | PADS_BANK0_GPIO1_IE_Msk | PADS_BANK0_GPIO1_SCHMITT_Msk; // Pullup = 1
    PADS_BANK0->GPIO8 = PADS_BANK0_GPIO1_PUE_Msk | PADS_BANK0_GPIO1_IE_Msk | PADS_BANK0_GPIO1_SCHMITT_Msk; // Pullup = 1
    delay_us(1);    // bei LCD-Routinen
// Zustand lesen    
    akt_tasten = (SIO->GPIO_IN >> TASTEN_OFFSET & TASTEN_MASKE) ^ TASTEN_MASKE; // GPIO 6,7,8
// fuer LCD wieder auf Ausgang
    SIO->GPIO_OE_SET = TASTEN_MASKE << TASTEN_OFFSET; // Ausgaenge aktivieren

// in 'tasten_schalter' muss das Eingangsbitmuster stehen hier nur PortA
    tasten_schalter = deko_tab[akt_tasten];
    if(rep_ctr) rep_ctr--;                    // repeat-counter dekrementieren
    entprell_ctr++;                           // Entprell-counter inkrementieren
    if(tasten_schalter != temp_tasten) {      // Aenderung der Bits ?
      temp_tasten = tasten_schalter;          // ja, speichern
      entprell_ctr=0;                         // und neu entprellen
      return;                                 // mehr nicht
    }

    if(entprell_ctr < ENTPRELL_ZEIT) return;  // Entprellzeit abwarten
    entprell_ctr=0;                           // und neu starten
    // Tasten sind entprellt
    // jetzt auf positive Aenderungen testen
    aenderung = (temp_tasten ^ entprellte_tasten) & temp_tasten;
    entprellte_tasten = temp_tasten; // und letzten Zustand merken

    if(aenderung) {                           // falls neue Tasten aktiviert
      letzte_aenderung = aenderung;           // Aenderung merken
      rep_ctr = FIRST_REPEAT;                 // und 1.Repeat aktivieren
      rep_flag = 0;
    }
    else if(rep_ctr == 0 && rep_flag) {       // keine neue Taste aber repeat aktiv
      aenderung = entprellte_tasten & letzte_aenderung; // letzte Aenderung
      rep_ctr = NEXT_REPEAT;                  // wiederholen, mit kurzer repeat-Zeit
    }
    n=0;                          // Auswertung der Tasten
    do {                          // bit suchen
      if(aenderung & 1) {         // bit gefunden
        if(aenderung == 1) {      // und nur eine Taste erlaubt
          if(!rep_flag) {         // kein repeat: normale Codes verwenden
            n = tast_tab[n];
            rep_flag = 1;         // aber beim naechsten Mal repeat
          } else {
            n = rep_tast_tab[n];  // repeat ist schon aktiv
          }
          if(n != ' ') taste = n; // nur gueltigen Code speichern
          else rep_flag = 0;      // kein repeat, wenn keine gueltige taste
        }
        return;                   // taste erkannt, dekodierung fertig
      }
      n++;
      aenderung >>= 1;
    } while(n < MAX_BITS);
    return;                       // keine taste gefunden
}

// Anpassung fr 20 Zeichen/Zeile
void relative_lcd_position(uint8_t zeile, uint8_t spalte)
{
uint8_t spalten_offset = (lcd_breite-MIN_LCD_BREITE)/2;  // ggf. 20 Zeichen
  lcd_zeile_spalte(zeile, spalte+spalten_offset);
}  

// PWM fr Kontrast verndern
int stelle_kontrast()
{
int c,i, repeat=0, inc_dec=1, abbruch=0;
int org_wert = lcd_kontrast;
char s[TEMP_STR_LEN];
  loesche_lcd_zeile12();
  lcd_zeile1("  LCD-Kontrast  ");
  i = lcd_kontrast;
  for(;;) {
    sprintf(s,"%3d",i);
    relative_lcd_position(2,7);
    lcd_string(s);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case PLUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_5;
	    if(i < MAX_LCD_KONTRAST) i += inc_dec;
        if(i >= MAX_LCD_KONTRAST) i = MAX_LCD_KONTRAST;
        setze_kontrast(i);
        break;
      case MINUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case MINUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_5;
	    if(i > MIN_LCD_KONTRAST) i -= inc_dec;
        if(i <= MIN_LCD_KONTRAST) i = MIN_LCD_KONTRAST;
        setze_kontrast(i);
        break;
      case PLUS_MINUS:
        setze_kontrast(org_wert);
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        if(i != org_wert) schreibe_kontrast(i);
	    return(abbruch);
    }
  }
}

// Blinkdauer von fertig-LED einstellen
int stelle_led_ein_zeit()
{
int c,i, repeat=0, inc_dec=1, abbruch = 0;
int org_wert = led_ein_zeit;
char s[TEMP_STR_LEN];
  loesche_lcd_zeile12();
  lcd_zeile1("  LED EIN-Zeit  ");
  i = led_ein_zeit;
  for(;;) {
    sprintf(s,"%5d",i);
    relative_lcd_position(2,5);
    lcd_string(s);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case PLUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i < MAX_LED_EIN_ZEIT) i += inc_dec;
        if(i >= MAX_LED_EIN_ZEIT) i = MAX_LED_EIN_ZEIT;
        led_ein_zeit = i;
        break;
      case MINUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case MINUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i > MIN_LED_EIN_ZEIT) i -= inc_dec;
        if(i <= MIN_LED_EIN_ZEIT) i = MIN_LED_EIN_ZEIT;
        led_ein_zeit = i;
        break;
      case PLUS_MINUS:
        led_ein_zeit = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        if(i != org_wert) schreibe_led_zeit(i);
	    return(abbruch);
    }
  }
}

// feste Werte aus Tabelle per baudraten_index auswhlen
int stelle_baudrate()
{
int c,i, max_i, abbruch = 0;
int org_wert = baudraten_index;
  i = org_wert;
  loesche_lcd_zeile12();
  lcd_zeile1(" ser. Baudrate ");
  max_i = sizeof(baud_tab)/sizeof(*baud_tab)-1;
  for(;;) {
    relative_lcd_position(2,6);
    lcd_string(baud_string[i]);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
	    if(i < max_i) {
	      i++;
	      set_baudrate1(baud_tab[i]);
	    }
	    break;
      case MINUS_TASTE:
	    if(i > 0) {
	      i--;
	      set_baudrate1(baud_tab[i]);
	    }
        break;
      case PLUS_MINUS:
        baudraten_index = org_wert;
        set_baudrate1(baud_tab[org_wert]);
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        if(i != org_wert) schreibe_baudraten_index(i);
	    return(abbruch);
    }
  }
}

int stelle_ser_ausgabe()
{
int c,i, max_i, abbruch = 0;
int org_wert = ser_ausgabe_kanal;
char *text[] = {"keine Ausgabe",
                "F1 Frequenz  ", 
                "F1 Periode  ", 
                "F1 Drehzahl  "};
  i = org_wert;
  max_i = MAX_SER_KANAL;
  if(i >= max_i) i = 0;
  loesche_lcd_zeile12();
  lcd_zeile1("ser. Ausgabewert");
  for(;;) {
    relative_lcd_position(2,3);
    lcd_string(text[i]);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
	    if(i < max_i-1) {
	      i++;
	      ser_ausgabe_kanal = i;
	    }
	    break;
      case MINUS_TASTE:
	    if(i > 0) {
	      i--;
	      ser_ausgabe_kanal = i;
	    }
        break;
      case PLUS_MINUS:
        ser_ausgabe_kanal = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        ee_write_uint8(EE_ADR_SER_AUSGABE, ser_ausgabe_kanal);
	    return(abbruch);
    }
  }
}

int stelle_anzeige_format()
{
int c,i, max_i, abbruch = 0;
int org_wert = anzeige_format;
char *text[] = {"1.23456789 Hz",
                "1.23456789E+0", 
                "1,23456789 Hz",
                "1,23456789E+0"};
  i = org_wert;
  max_i = sizeof(text)/sizeof(*text)-1;
  if(i > max_i) i = 0;
  loesche_lcd_zeile12();
  lcd_zeile1(" Anzeigeformat  ");
  for(;;) {
    relative_lcd_position(2,2);
    lcd_string(text[i]);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
	    if(i < max_i) {
	      i++;
	      anzeige_format = i;
	    }
	    break;
      case MINUS_TASTE:
	    if(i > 0) {
	      i--;
	      anzeige_format = i;
	    }
        break;
      case PLUS_MINUS:
        anzeige_format = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        ee_write_uint8(EE_ADR_ANZEIGE_FORMAT, anzeige_format);
	    return(abbruch);
    }
  }
}



// Korrekturwert manuell einstellen: lokale Fref
int manueller_abgleich_intern()
{
int c,i, repeat=0, inc_dec=1, abbruch = 0;
int org_wert = Fintern_offset;
char s[TEMP_STR_LEN];
  loesche_lcd_zeile12();
  lcd_zeile1(" manu. Abgleich ");
  i = org_wert;
  for(;;) {
  sprintf(s," intern:%7d",i);
    lcd_zeile2(s);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case PLUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
        if(repeat >= REPEAT_59) inc_dec = INC_DEC_1000;
	    if(i < MAX_OFFSET) i += inc_dec;
        if(i >= MAX_OFFSET) i = MAX_OFFSET;
        Fintern_offset = i;
        if(ex_ref_aktiv == 0) {        // nderung nur durchfhren, wenn int. Ref. aktiv
          F1_offset = i;
          setze_F1_offset();
        }  
        break;
      case MINUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case MINUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
        if(repeat >= REPEAT_59) inc_dec = INC_DEC_1000;
	    if(i > MIN_OFFSET) i -= inc_dec;
        if(i <= MIN_OFFSET) i = MIN_OFFSET;
        Fintern_offset = i;
        if(ex_ref_aktiv == 0) {       // nderung nur durchfhren, wenn int. Ref. aktiv
          F1_offset = i;
          setze_F1_offset();
        }  
        break;
      case PLUS_MINUS:
        Fintern_offset = org_wert;
        if(ex_ref_aktiv == 0) {       // nderung nur durchfhren, wenn int. Ref. aktiv
          F1_offset = org_wert;
          setze_F1_offset();
        }  
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        Fintern_offset = i;
        ee_write_uint32(EE_ADR_Fintern_OFFSET, Fintern_offset);
	    return(abbruch);
    }
  }
}

// Korrekturwert manuell einstellen: externe Fref
int manueller_abgleich_extern()
{
int c,i, repeat=0, inc_dec=1, abbruch = 0;
int org_wert = Fextern_offset;
char s[TEMP_STR_LEN];
  loesche_lcd_zeile12();
  lcd_zeile1(" manu. Abgleich ");
  i = org_wert;
  for(;;) {
  sprintf(s," extern:%7d",i);
    lcd_zeile2(s);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case PLUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
        if(repeat >= REPEAT_59) inc_dec = INC_DEC_1000;
	    if(i < MAX_OFFSET) i += inc_dec;
        if(i >= MAX_OFFSET) i = MAX_OFFSET;
        Fextern_offset = i;
        if(ex_ref_aktiv == 1) {       // nderung nur durchfhren, wenn ext. Ref. aktiv
          F1_offset = i;
          setze_F1_offset();
        }  
        break;
      case MINUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case MINUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
        if(repeat >= REPEAT_59) inc_dec = INC_DEC_1000;
	    if(i > MIN_OFFSET) i -= inc_dec;
        if(i <= MIN_OFFSET) i = MIN_OFFSET;
        Fextern_offset = i;
        if(ex_ref_aktiv == 1) {       // nderung nur durchfhren, wenn ext. Ref. aktiv
          F1_offset = i;
          setze_F1_offset();
        }  
        break;
      case PLUS_MINUS:
        Fextern_offset = org_wert;
        if(ex_ref_aktiv == 1) {       // nderung nur durchfhren, wenn ext. Ref. aktiv
          F1_offset = org_wert;
          setze_F1_offset();
        }  
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        Fextern_offset = i;
        ee_write_uint32(EE_ADR_Fextern_OFFSET, Fextern_offset);
	    return(abbruch);
    }
  }
}

// an vorhandene Anzeige anpassen
int lcd_breite_16_20()
{
int c,i, abbruch = 0;
int org_wert = lcd_breite;
char *ein_aus[] = {"   16 Zeichen", "   20 Zeichen"};
  i = org_wert/20;    // 0 oder 1
  for(;;) {
    loesche_lcd_zeile12();
    lcd_zeile1("   LCD-Breite");
    lcd_zeile2(ein_aus[i]);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
        if(i == 1) break;
        i=1;
        lcd_breite = 20;
        init_zeilen_breite(lcd_breite);
        loesche_lcd_zeile34();
	    break;
      case MINUS_TASTE:
        if(i == 0) break;
        i=0;
        loesche_lcd_zeile34();
        lcd_breite = 16;
        init_zeilen_breite(lcd_breite);
        break;
      case PLUS_MINUS:
        lcd_breite = org_wert;
        init_zeilen_breite(lcd_breite);
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        if(lcd_breite != org_wert) {
          loesche_lcd_zeile34();
          ee_write_uint8(EE_ADR_LCD_BREITE, lcd_breite);
        }  
	    return(abbruch);
    }
  }
}

// spezielles Format fr Messwertanzeige
int MHz_on_off()
{
int c,i, abbruch = 0;
int org_wert = nur_MHz;
char *ein_aus[] = {"normal ", "nur MHz"};
  i = org_wert;
  loesche_lcd_zeile12();
  lcd_zeile1(" Frequenzanzeige ");
  for(;;) {
    relative_lcd_position(2,6);
    lcd_string(ein_aus[i]);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
      case MINUS_TASTE:
        i ^= 1;
        break;
      case PLUS_MINUS:
        nur_MHz = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        nur_MHz = i;
        if(i != org_wert) {
          ee_write_uint8(EE_ADR_NUR_MHZ, nur_MHz);
        }  
 	    return(abbruch);
    }
  }
}


// Grundeinstellungen
void menue_allgemein()
{
static uint8_t marke;
  for(;;) {
    switch(marke) {
      case(0):
        if(stelle_ser_ausgabe()) return;
        marke++;
        break;
      case(1):
        if(stelle_led_ein_zeit()) return;
        marke++;
        break;
      case(2):      
        if(stelle_baudrate()) return;
        marke++;
        break;
      case(3):      
        if(manueller_abgleich_intern()) return;
        marke ++;
        break;
      case(4):      
        if(manueller_abgleich_extern()) return;
        marke++;
        break;
      case(5):      
        if(lcd_breite_16_20()) return;
        marke++;
        break;
      case(6):      
        if(stelle_kontrast()) return;
        marke++;
        break;
      case(7):
        if(stelle_anzeige_format()) return;
        marke++;
        break;
      case(8):
        if(MHz_on_off()) return;
        marke++;
        break;
      default:
        marke = 0;    // wieder auf Anfang
    }
  }
}

// Bedienmenue fr manuelle Einstellungen
void FM_AS65_hauptmenue()
{
static uint8_t key, temp;
#define MAX_MENUE 2
char *text[] = {"   Eingang F1   ","   allgemein    "};
static void (*menue_funktion[])(void) = {menue_F1, menue_allgemein};

  loesche_lcd_zeile12();
  do {
    lcd_zeile1(" Einstellungen  ");
    lcd_zeile2(text[key]);
    while((temp = lese_taste()) == 0);
    switch(temp) {
      case PLUS_TASTE:
        key++;
        if(key >= MAX_MENUE) key = 0;
      break;
      case MINUS_TASTE:
        if(key) key--;
        else key = MAX_MENUE-1;
      break;
      case INIT_TASTE:
        menue_funktion[key]();      // angewhltes Menue aufrufen
      case ABBRUCH_TASTE:
      case PLUS_MINUS:
        return;
    }
  }while(1);
}  
      
int stelle_F1_messzeit()
{
int c,i, repeat=0, inc_dec=1, abbruch = 0;
int org_wert = F1_messzeit;
char s[TEMP_STR_LEN];
  loesche_lcd_zeile12();
  lcd_zeile1("  F1 Messzeit  ");
  i = org_wert;
  for(;;) {
    sprintf(s,"%5d",i);
    relative_lcd_position(2,5);
    lcd_string(s);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case PLUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i < MAX_MESSZEIT) i += inc_dec;
        if(i >= MAX_MESSZEIT) i = MAX_MESSZEIT;
        F1_messzeit = i;
        break;
      case MINUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case MINUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i > MIN_MESSZEIT) i -= inc_dec;
        if(i <= MIN_MESSZEIT) i = MIN_MESSZEIT;
        F1_messzeit = i;
        break;
      case PLUS_MINUS:
        F1_messzeit = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        if(i != org_wert) {
          F1_messzeit = i;
          ee_write_uint32(EE_ADR_F1_MESSZEIT, F1_messzeit);
        } 
	    return(abbruch);
    }
  }
}
   

   
int stelle_F1_timeout()
{
int c,i, repeat=0, inc_dec=1, abbruch = 0;
int org_wert = F1_timeout;
char s[TEMP_STR_LEN];
  loesche_lcd_zeile12();
  lcd_zeile1("  F1 Timeout  ");
  i = org_wert;
  for(;;) {
    sprintf(s,"%5d",i);
    relative_lcd_position(2,5);
    lcd_string(s);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case PLUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i < MAX_MESSZEIT) i += inc_dec;
        if(i >= MAX_MESSZEIT) i = MAX_MESSZEIT;
        F1_timeout = i;
        break;
      case MINUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case MINUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i > MIN_MESSZEIT) i -= inc_dec;
        if(i <= MIN_MESSZEIT) i = MIN_MESSZEIT;
        F1_timeout = i;
        break;
      case PLUS_MINUS:
        F1_timeout = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        if(i != org_wert) {
          F1_timeout = i;
          ee_write_uint32(EE_ADR_F1_TIMEOUT, F1_timeout);
        } 
	    return(abbruch);
    }
  }
}

// fr Vorteiler, falls verwendet
int stelle_F1_faktor()
{
int c,i, repeat=0, inc_dec=1, abbruch = 0;
int org_wert = F1_faktor;
char s[TEMP_STR_LEN];
  loesche_lcd_zeile12();
  lcd_zeile1("   F1 Faktor  ");
  i = org_wert;
  for(;;) {
    sprintf(s,"%5d",i);
    relative_lcd_position(2,5);
    lcd_string(s);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case PLUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i < MAX_FAKTOR) i += inc_dec;
        if(i >= MAX_FAKTOR) i = MAX_FAKTOR;
        F1_faktor = i;
        break;
      case MINUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case MINUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i > MIN_FAKTOR) i -= inc_dec;
        if(i <= MIN_FAKTOR) i = MIN_FAKTOR;
        F1_faktor = i;
        break;
      case PLUS_MINUS:
        F1_faktor = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        if(i != org_wert) {
          F1_faktor = i;
          ee_write_uint32(EE_ADR_F1_FAKTOR, F1_faktor);
        } 
	    return(abbruch);
    }
  }
}
  
// nur bei UPM aktiv
int stelle_F1_upm_divisor()
{
int c,i, repeat=0, inc_dec=1, abbruch = 0;
int org_wert = F1_upm_divisor;
char s[TEMP_STR_LEN];
  loesche_lcd_zeile12();
  lcd_zeile1(" F1 UPM-Divisor");
  i = org_wert;
  for(;;) {
    sprintf(s,"%5d",i);
    relative_lcd_position(2,5);
    lcd_string(s);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case PLUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i < MAX_UPM_DIVISOR) i += inc_dec;
        if(i >= MAX_UPM_DIVISOR) i = MAX_UPM_DIVISOR;
        F1_upm_divisor = i;
        break;
      case MINUS_TASTE:
        inc_dec = INC_DEC_1;        // Startwert fr +/-1
        repeat = 0;
      case MINUS_REPEAT:  
        repeat++;
        if(repeat >= REPEAT_19) inc_dec = INC_DEC_10;
        if(repeat >= REPEAT_39) inc_dec = INC_DEC_100;
	    if(i > MIN_UPM_DIVISOR) i -= inc_dec;
        if(i <= MIN_UPM_DIVISOR) i = MIN_UPM_DIVISOR;
        F1_upm_divisor = i;
        break;
      case PLUS_MINUS:
        F1_upm_divisor = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        if(i != org_wert) {
          F1_upm_divisor = i;
          ee_write_uint32(EE_ADR_F1_UPM_DIV, F1_upm_divisor);
        } 
	    return(abbruch);
    }
  }
}
   
int stelle_F1_stellen()
{
int c,i, abbruch = 0;
int org_wert = F1_stellen;
char s[TEMP_STR_LEN];
  loesche_lcd_zeile12();
  lcd_zeile1("  F1 Stellen  ");
  i = org_wert;
  for(;;) {
    
    if(i) {                               // bei fester Stellenzahl
      sprintf(s,"  %5d     ",i);
      relative_lcd_position(2,2);
      lcd_string(s);
    }  
    else lcd_zeile2("  automatisch");     // bei Stellenanzahl == 0
    

    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
	    if(i < F1_MAX_STELLEN) i++;
        if(i < MIN_STELLEN) i = MIN_STELLEN;
        if(i >= F1_MAX_STELLEN) i = F1_MAX_STELLEN;
        F1_stellen = i;
        break;
      case MINUS_TASTE:
	    if(i >= MIN_STELLEN) i--;
        if(i < MIN_STELLEN) i = 0;   // automatisch
        F1_stellen = i;
        break;
      case PLUS_MINUS:
        F1_stellen = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        if(i != org_wert) {
          F1_stellen = i;
          ee_write_uint32(EE_ADR_F1_STELLEN, F1_stellen);
        } 
	    return(abbruch);
    }
  }
}
   
// Multiplikator einschalten
int F1_vorteiler_on_off()
{
int c,i, abbruch = 0;
int org_wert = F1_vorteiler_aktiv;
char *ein_aus[] = {" aus ", "aktiv"};
  i = org_wert;
  loesche_lcd_zeile12();
  lcd_zeile1("  F1 Vorteiler  ");
  for(;;) {
    relative_lcd_position(2,6);
    lcd_string(ein_aus[i]);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
      case MINUS_TASTE:
        i ^= 1;
        break;
      case PLUS_MINUS:
        F1_vorteiler_aktiv = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        F1_vorteiler_aktiv = i;
        if(i != org_wert) {
          ee_write_uint8(EE_ADR_F1_VORTEILER, F1_vorteiler_aktiv);
        }  
	    return(abbruch);
    }
  }
}

// Multiplikator einschalten
int F1_vorteiler_umschaltung()
{
int c,i, abbruch = 0;
int org_wert = auto_vorteiler_umschaltung;
char *ein_aus[] = {" weiter messen  ", "    Neustart    "};
  i = org_wert;
  loesche_lcd_zeile12();
  lcd_zeile1("F1 +/- Vorteiler");
  for(;;) {
    relative_lcd_position(2,1);
    lcd_string(ein_aus[i]);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
      case MINUS_TASTE:
      i ^= 1;
      break;
      case PLUS_MINUS:
        auto_vorteiler_umschaltung = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        auto_vorteiler_umschaltung = i;
        if(i != org_wert) {
          ee_write_uint8(EE_ADR_F1_AUTO_VORTEILER, auto_vorteiler_umschaltung);
        }  
	    return(abbruch);
    }
  }
}

int F1_regression_on_off()
{
int c,i, abbruch = 0;
int org_wert = F1_reg_modus;
char *ein_aus[] = {" aus ", "aktiv"};
  if(org_wert > OPTIMIERT) org_wert = OPTIMIERT;
  i = org_wert;
  loesche_lcd_zeile12();
  lcd_zeile1      ("lin. Regression");
  for(;;) {
    relative_lcd_position(2,6);
    lcd_string(ein_aus[i]);
    while((c=lese_taste()) == 0);
    switch(c) {
      case PLUS_TASTE:
	      i++;
        if(i > OPTIMIERT) i = 0;
	    break;
      case MINUS_TASTE:
	      i--;
        if(i < 0) i = OPTIMIERT;
        break;
      case PLUS_MINUS:
        F1_reg_modus = org_wert;
        return(1);                    // Abbruch ohne nderungen
      case ABBRUCH_TASTE:
	    abbruch = 1;
      case INIT_TASTE:
        F1_reg_modus = i;
        if(i != org_wert) {
          ee_write_uint8(EE_ADR_F1_REGRESSION, F1_reg_modus);
        }  
 	    return(abbruch);
    }
  }
}

// manuelle Einstellungen fr Kanal F1    
void menue_F1()
{
static uint8_t marke;
  for(;;) {
    switch(marke) {
      case(0):
        if(stelle_F1_messzeit()) return;
        marke++;
        break;
      case(1):
        if(stelle_F1_timeout()) return;
        marke++;
        break;
      case(2):      
        if(stelle_F1_stellen()) return;
        marke++;
        break;
      case(3):      
        if(F1_vorteiler_umschaltung()) return;
        marke++;
        break;
      case(4):      
        if(F1_vorteiler_on_off()) return;
        if(!F1_vorteiler_aktiv) marke++;   // Vorteiler bei Bedarf
        marke++;
        break;
      case(5):      
        if(stelle_F1_faktor()) return;
        marke++;
        break;
      case(6):      
        if(stelle_F1_upm_divisor()) return;
        marke++;
        break;
      case(7):      
        if(F1_regression_on_off()) return;
        marke++;
        break;
      default:
        marke = 0;    // wieder auf Anfang
    }
  }
}
